@@ -219,14 +219,22 @@ function install_bootstrap()
 
 function pin_newer_runtime_libs ()
 {
+	# First argument is the runtime path
+	steam_runtime_path=$(realpath "$1")
+
+	if [[ ! -d "$steam_runtime_path" ]]; then return; fi
+
+	# Try to use the Steam Runtime's own logic instead of our own copy
+	if [ -x "$steam_runtime_path/setup.sh" ]; then
+		"$steam_runtime_path/setup.sh" --force
+		return $?
+	fi
+	# Remainder is backwards compatibility with older Steam Runtime
+	# versions, before they absorbed this logic
+
 	# Set separator to newline just for this function
 	local IFS=$(echo -en "\n\b")
-	
-	# First argument is the runtime path
-	steam_runtime_path=`realpath $1`
-	
-	if [[ ! -d "$steam_runtime_path" ]]; then return; fi
-	
+
 	# Associative array; indices are the SONAME, values are final path
 	declare -A host_libraries_32
 	declare -A host_libraries_64
@@ -381,14 +389,22 @@ function pin_newer_runtime_libs ()
 
 function check_pins ()
 {
+	# First argument is the runtime path
+	steam_runtime_path=$(realpath "$1")
+
+	if [[ ! -d "$steam_runtime_path" ]]; then return; fi
+
+	# Try to use the Steam Runtime's own logic instead of our own copy
+	if [ -x "$steam_runtime_path/setup.sh" ]; then
+		"$steam_runtime_path/setup.sh"
+		return $?
+	fi
+	# Remainder is backwards compatibility with older Steam Runtime
+	# versions, before they absorbed this logic
+
 	# Set separator to newline just for this function
 	local IFS=$(echo -en "\n\b")
 	
-	# First argument is the runtime path
-	steam_runtime_path=`realpath $1`
-	
-	if [[ ! -d "$steam_runtime_path" ]]; then return; fi
-	
 	pins_need_redoing="no"
 	
 	# If we had the runtime previously unpacked but never ran the pin code, do it now
@@ -816,16 +832,17 @@ fi
 if [ "$STEAM_RUNTIME" -a "$STEAM_RUNTIME" != "0" ]; then
 	# Unpack the runtime if necessary
 	if unpack_runtime; then
-		case $(uname -m) in
-			*64)
-				export PATH="$STEAM_RUNTIME/amd64/bin:$STEAM_RUNTIME/amd64/usr/bin:$PATH"
-				;;
-			*)
-				export PATH="$STEAM_RUNTIME/i386/bin:$STEAM_RUNTIME/i386/usr/bin:$PATH"
-				;;
-		esac
-		
-		export STEAM_RUNTIME_LIBRARY_PATH=`$STEAM_RUNTIME/run.sh --print-steam-runtime-library-paths`
+		if [ ! -x "$STEAM_RUNTIME/setup.sh" ]; then
+		    echo "internal error: $STEAM_RUNTIME/setup.sh is missing, this runtime is invalid or corrupted"
+		    exit 1
+		fi
+		export PATH="$("$STEAM_RUNTIME/setup.sh" --print-bin-path):$PATH"
+
+		if [ ! -x "$STEAM_RUNTIME/run.sh" ]; then
+		    echo "internal error: $STEAM_RUNTIME/run.sh is missing, this runtime is invalid or corrupted"
+		    exit 1
+		fi
+		export STEAM_RUNTIME_LIBRARY_PATH="$("$STEAM_RUNTIME/run.sh" --print-steam-runtime-library-paths)"
 
 		export LD_LIBRARY_PATH="$STEAM_RUNTIME_LIBRARY_PATH:${LD_LIBRARY_PATH-}"
 	else
